/**
* The MIT License
*
* Copyright (c) 2013 saka-en, http://www.saka-en.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/
package datepicker
{
	import flash.events.MouseEvent;

	import mx.collections.ArrayCollection;
	import mx.containers.HBox;
	import mx.containers.VBox;
	import mx.controls.Text;

	/**
	 * DatePicker クラス
	 */
	public class DatePicker
	{
		/**
		 * 開始日
		 */
		private var _startDate:Date;
		/**
		 * 開始日の年
		 */
		private var _year:int;
		/**
		 * 開始日の月
		 */
		private var _month:int;
		/**
		 * 平日リスト
		 */
		private var _weekdayList:ArrayCollection;
		/**
		 * 休日リスト
		 */
		private var _holidayList:ArrayCollection;
		/**
		 * 土曜日休日
		 */
		private var _saturdayIsHoliday:Boolean;
		/**
		 * 日曜日休日
		 */
		private var _sundayIsHoliday:Boolean;
		/**
		 * 利用可日付のリスト
		 */
		private var _enabledList:ArrayCollection;
		/**
		 * 利用不可日付のリスト
		 */
		private var _disabledList:ArrayCollection;

		/**
		 * コンストラクタ
		 *
		 * @param startDate 開始日(月の初日が設定されている必要がある。)
		 */
		public function DatePicker(startDate:Date) {
			_startDate = startDate;
			_year = startDate.fullYear;
			_month = startDate.month;
			_enabledList = new ArrayCollection;
			_disabledList = new ArrayCollection;
			_weekdayList = new ArrayCollection;
			_holidayList = new ArrayCollection;
		}

		/**
		 * カレンダー作成
		 */
		public function creation():VBox {
			// どちらのリストにも設定がある場合、利用不可日付リストをクリア
			if ( _weekdayList.length > 0 && _holidayList.length > 0 )
				_holidayList = new ArrayCollection;
			// 月の最終日を取得
			var endDate:Date = Util.lastDayOfThisMonth(_startDate);
			// 一番外側のボックス
			var mainBox:VBox = new VBox;
			mainBox.id = "m" + _year.toString();
			mainBox.styleName = "contentsCalendarSpace";
			// タイトル行のボックス作成
			mainBox.addChild(makeTitleBox());
			// 週タイトル(日～土)のボックス作成
			mainBox.addChild(makeWeekBox());
			var row:int = 1;
			var pos:int = 0;
			var hgrp:HBox;
			for ( var i:int = 0; i < endDate.date; i++ ) {
				// 日付を生成
				var date:Date = Util.toDate(_year.toString() + "/" + (_month + 1).toString() + "/" + (i + 1).toString() );
				if ( date == null ) break;
				// 曜日を取得
				var week:int = date.day;
				if ( pos == 0 ) {
					// 週の日付ボックス(7日分）
					hgrp = new HBox;
					hgrp.styleName = "contentsCalendarSpace";
					// 開始位置を設定
					if ( row == 1 ) {
						var startWeek:int = _startDate.day;
						for ( var x:int = 0; x < startWeek; x++ ) {
							var spBox:HBox = new HBox;
							spBox.styleName = "contentsCalendarSpace";
							spBox.width = 30;
							hgrp.addChild(spBox);
						}
					}
					pos = week;
				}
				// 日のボックス
				var hbox:HBox = makeDayBox(date, week, pos);
				// 日テキスト
				var text:Text = makeDayText(date, week, pos);
				hbox.addChild(text);
				hgrp.addChild(hbox);
				mainBox.addChild(hgrp);
				pos += 1;
				if ( pos == 7 ) {
					row += 1;
					pos = 0;
				}
			}
			return mainBox;
		}

		/**
		 * タイトル行ボックスの作成
		 */
		private function makeTitleBox():HBox {
			// タイトル行のボックス
			var titleBox:HBox = new HBox;
			titleBox.styleName = "contentsCalendarTitle";
			titleBox.width = 215;
			//var link:LinkButton = new LinkButton;	// リンクボタンはやめてテキストに
			//link.styleName = "contentsLink";
			//link.label = (_month + 1) + MessageUtils.getString("label.month");
			var link:Text = new Text;
			link.styleName = "contentsCalendarTitleText";
			link.text = (_month + 1) + "月";
			titleBox.addChild(link);
			return titleBox;
		}

		/**
		 * 週タイトル(日～土)のボックスの作成
		 */
		private function makeWeekBox():HBox {
			// 週タイトル(日～土)のボックス
			var weekBox:HBox = new HBox;
			weekBox.styleName = "contentsCalendarSpace";
			for ( var t:int = 0; t < 7; t++ ) {
				var firstWeek:Date = new Date(2012, 6, t + 1);	//TODO 日曜から始まればなんでもよいけどダサいか
				var w:int = firstWeek.day;
				var title:Text = new Text;
				title.id = "m"+(_startDate.month + 1)+w;
				title.text = Util.getDateOfTheWeek(firstWeek);
				title.width = 30;
				title.styleName = "contentsLabelCalendarWeek";
				weekBox.addChild(title);
			}
			return weekBox;
		}

		/**
		 * 日のボックス作成
		 *
		 * @param date 作成したい日付
		 * @param week 作成したい日付の曜日
		 * @param pos 表示位置
		 */
		private function makeDayBox(date:Date, week:int, pos:int):HBox {
			// 日のボックス
			var hbox:HBox = new HBox;
			hbox.id = "mB"+(date.month + 1)+pos+week;
			hbox.width = 30;
			hbox.height = 20;
			// 平日の設定がある場合
			if ( _weekdayList.length > 0 ) {
				if ( isEquals(date, _weekdayList)  ) {
					// 土曜日休暇の場合
					if ( week == 6 && _saturdayIsHoliday ) {
						_disabledList.addItem(date);
						hbox.styleName = "contentsTextDisabledCalendar";
					// 日曜日休暇の場合
					} else if ( week == 0 && _sundayIsHoliday ) {
						_disabledList.addItem(date);
						hbox.styleName = "contentsTextDisabledCalendar";
					} else {
						_enabledList.addItem(date);
						hbox.styleName = "contentsTextCalendar";
					}
				} else {
					_disabledList.addItem(date);
					hbox.styleName = "contentsTextDisabledCalendar";
				}
			// 休日の設定がある場合
			} else if ( _holidayList.length > 0 ) {
				if ( isEquals(date, _holidayList)  ) {
					_disabledList.addItem(date);
					hbox.styleName = "contentsTextDisabledCalendar";
				} else {
					// 土曜日休暇の場合
					if ( week == 6 && _saturdayIsHoliday ) {
						_disabledList.addItem(date);
						hbox.styleName = "contentsTextDisabledCalendar";
					// 日曜日休暇の場合
					} else if ( week == 0 && _sundayIsHoliday ) {
						_disabledList.addItem(date);
						hbox.styleName = "contentsTextDisabledCalendar";
					} else {
						_enabledList.addItem(date);
						hbox.styleName = "contentsTextCalendar";
					}
				}
			} else {
				// 土曜日休暇の場合
				if ( week == 6 && _saturdayIsHoliday ) {
					_disabledList.addItem(date);
					hbox.styleName = "contentsTextDisabledCalendar";
				// 日曜日休暇の場合
				} else if ( week == 0 && _sundayIsHoliday ) {
					_disabledList.addItem(date);
					hbox.styleName = "contentsTextDisabledCalendar";
				}
			}
			// 設定がない場合
			if ( hbox.styleName == null ) {
				_enabledList.addItem(date);
				hbox.styleName = "contentsTextCalendar";
			}
			hbox.addEventListener(MouseEvent.MOUSE_OVER,
				function (mouseEvent:MouseEvent):void {
					mouseOverHandler(mouseEvent);
				}
			);
			hbox.addEventListener(MouseEvent.MOUSE_OUT,
				function (mouseEvent:MouseEvent):void {
					mouseOutHandler(mouseEvent);
				}
			);
			hbox.addEventListener(MouseEvent.MOUSE_DOWN,
				function (mouseEvent:MouseEvent):void {
					mouseDownHandler(mouseEvent);
				}
			);
			return hbox;
		}

		/**
		 * 日のテキスト作成
		 *
		 * @param date 作成したい日付
		 * @param week 作成したい日付の曜日
		 * @param pos 表示位置
		 */
		private function makeDayText(date:Date, week:int, pos:int):Text {
			var text:Text = new Text;
			text.id = "mT"+(date.month + 1)+pos+week;
			text.text = date.date.toString();
			text.width = 30;
			text.height = 20;
			// 平日の設定がある場合
			if ( _weekdayList.length > 0 ) {
				if ( isEquals(date, _weekdayList)  )
					text.styleName = "contentsTextCalendar";
				else
					text.styleName = "contentsTextDisabledCalendar";
			// 休日の設定がある場合
			} else if ( _holidayList.length > 0 ) {
				if ( isEquals(date, _holidayList)  )
					text.styleName = "contentsTextDisabledCalendar";
				else
					text.styleName = "contentsTextCalendar";
			}
			// 土曜日休暇の場合
			if ( week == 6 && _saturdayIsHoliday ) {
				text.styleName = "contentsTextDisabledCalendar";
			// 日曜日休暇の場合
			} else if ( week == 0 && _sundayIsHoliday ) {
				text.styleName = "contentsTextDisabledCalendar";
			}
			// 設定がない場合
			if ( text.styleName == null )
				text.styleName = "contentsTextCalendar";
			// ボタンのように見せ、マウスカーソルを変更する。
			text.buttonMode = true;
			text.useHandCursor = true;
			text.mouseChildren = false;
			text.addEventListener(MouseEvent.MOUSE_OVER,
				function (mouseEvent:MouseEvent):void {
					mouseOverHandler(mouseEvent);
				}
			);
			text.addEventListener(MouseEvent.MOUSE_OUT,
				function (mouseEvent:MouseEvent):void {
					mouseOutHandler(mouseEvent);
				}
			);
			text.addEventListener(MouseEvent.MOUSE_DOWN,
				function (mouseEvent:MouseEvent):void {
					mouseDownHandler(mouseEvent);
					var data:Date = new Date(_year, _month, parseInt(mouseEvent.currentTarget.text));
					var idx:int = 0;
					if ( Util.equals(mouseEvent.currentTarget.styleName, "contentsTextDisabledCalendar") ) {
						// 選択不可日付に設定
						_disabledList.addItem(data);
						// 選択可日付から削除
						for each ( var enaObj:Object in _enabledList ) {
							var ena:Date = enaObj as Date;
							if ( data.time == ena.time ) {
								_enabledList.removeItemAt(idx);
								break;
							}
							idx++;
						}
					} else {
						// 選択可日付に設定
						_enabledList.addItem(data);
						// 選択不可日付から削除
						for each ( var disObj:Object in _disabledList ) {
							var dis:Date = disObj as Date;
							if ( data.time == dis.time ) {
								_disabledList.removeItemAt(idx);
								break;
							}
							idx++;
						}
					}
				}
			);
			return text;
		}

		/**
		 * マウスオーバハンドラ
		 *
		 * @param mouseEvent MouseEvent
		 */
		private function mouseOverHandler(mouseEvent:MouseEvent):void {
			if ( Util.equals(mouseEvent.currentTarget.styleName, "contentsTextDisabledCalendar") )
				return;
			if ( Util.equals(mouseEvent.currentTarget.styleName, "contentsTextCalendar") )
				mouseEvent.currentTarget.styleName = "contentsTextCalendarCursor";
		}

		/**
		 * マウスアウトハンドラ
		 *
		 * @param mouseEvent MouseEvent
		 */
		private function mouseOutHandler(mouseEvent:MouseEvent):void {
			if ( Util.equals(mouseEvent.currentTarget.styleName, "contentsTextDisabledCalendar") )
				return;
			if ( Util.equals(mouseEvent.currentTarget.styleName, "contentsTextCalendarCursor") )
				mouseEvent.currentTarget.styleName = "contentsTextCalendar";
		}

		/**
		 * マウス押下ハンドラ
		 *
		 * @param mouseEvent MouseEvent
		 */
		private function mouseDownHandler(mouseEvent:MouseEvent):void {
			if ( Util.equals(mouseEvent.currentTarget.styleName, "contentsTextCalendarCursor") )
				mouseEvent.currentTarget.styleName = "contentsTextDisabledCalendar";
			else
				mouseEvent.currentTarget.styleName = "contentsTextCalendar";
		}

		/**
		 * リスト内に同一データがあるかチェック
		 *
		 * @param date Date
		 * @param list ArrayCollection
		 */
		private function isEquals(date:Date, list:ArrayCollection):Boolean {
			for ( var i:int = 0; i < list.length; i++ ) {
				var data:Date = list.getItemAt(i) as Date;
				var tmp:Date = new Date(data.fullYear, data.month, data.date);
				if ( date.time  == tmp.time  )
					return true;
			}
			return false;
		}

		/**
		 * 平日リスト設定
		 *
		 * @param weekdayList 平日リスト
		 */
		public function set weekdayList(weekdayList:ArrayCollection):void {
			if ( weekdayList != null )
				_weekdayList = weekdayList;
		}

		/**
		 * 休日リスト設定
		 *
		 * @param holidayList 休日リスト
		 */
		public function set holidayList(holidayList:ArrayCollection):void {
			if ( holidayList != null )
				_holidayList = holidayList;
		}

		/**
		 * 土曜日休暇設定
		 *
		 * @param saturdayIsHoliday true:休暇, false:稼働
		 */
		public function set saturdayIsHoliday(saturdayIsHoliday:Boolean):void {
			_saturdayIsHoliday = saturdayIsHoliday;
		}

		/**
		 * 日曜日休暇設定
		 *
		 * @param sundayIsHoliday true:休暇, false:稼働
		 */
		public function set sundayIsHoliday(sundayIsHoliday:Boolean):void {
			_sundayIsHoliday = sundayIsHoliday;
		}

		/**
		 * 利用可能日リスト取得
		 */
		public function get enabledList():ArrayCollection {
			return _enabledList;
		}

		/**
		 * 利用不可能日リスト取得
		 */
		public function get disabledList():ArrayCollection {
			return _disabledList;
		}
	}
}