/**
* The MIT License
*
* Copyright (c) 2013 saka-en, http://www.saka-en.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/
package datepicker
{
	import flash.display.DisplayObject;

	import mx.collections.ArrayCollection;
	import mx.containers.HBox;
	import mx.containers.VBox;

	/**
	 * １ヶ月カレンダー
	 */
	public class OneMonth extends VBox
	{
		/**
		 * カレンダー
		 */
		private var _calendar:DatePicker;
		/**
		 * 平日リスト
		 */
		private var _weekdayList:ArrayCollection;
		/**
		 * 休日リスト
		 */
		private var _holidayList:ArrayCollection;
		/**
		 * 土曜日休暇
		 */
		private var _saturdayIsHoliday:Boolean;
		/**
		 * 日曜日休暇
		 */
		private var _sundayIsHoliday:Boolean;

		/**
		 * コンストラクタ
		 */
		public function OneMonth()
		{
			super();
			super.styleName = "contentsCalendarMainBox";

			// １ヶ月目のデフォルト設定
			_weekdayList = new ArrayCollection;
			_holidayList = new ArrayCollection;
			_saturdayIsHoliday = true;
			_sundayIsHoliday = true;
		}

		/**
		 * カレンダーの作成
		 *
		 * @param startDate 開始日
		 */
		public function creation(startDate:Date):void {
			remove();
			// カレンダーを設定
			var firstBox:HBox = new HBox;
			firstBox.name = "_calendarFirstBox";
			firstBox.setStyle("horizontalGap", 2);
			// カレンダー作成
			_calendar = new DatePicker(startDate);
			var today:Date = new Date;
			_calendar.weekdayList = _weekdayList;
			_calendar.holidayList = _holidayList;
			_calendar.saturdayIsHoliday = _saturdayIsHoliday;
			_calendar.sundayIsHoliday = _sundayIsHoliday;
			firstBox.addChild(_calendar.creation());
			this.addChild(firstBox);
		}

		/**
		 * オブジェクトの削除
		 */
		public function remove():void {
			var obj:Object = getChildByName("_calendarFirstBox");
			if ( obj != null )
				this.removeChild(obj as DisplayObject);
			obj = getChildByName("_calendarSecondBox");
			if ( obj != null )
				this.removeChild(obj as DisplayObject);
		}

		/**
		 * 利用可能日リスト取得
		 */
		public function getEnabledList():ArrayCollection {
			var enabledList:ArrayCollection = new ArrayCollection;
			var i:int = 0;
			var date:Date;
			// １か月目
			for ( i = 0; i < _calendar.enabledList.length; i++ ) {
				date = _calendar.enabledList.getItemAt(i) as Date;
				enabledList.addItem(date);
			}
			return enabledList;
		}

		/**
		 * 利用不可能日リスト取得
		 */
		public function getDisabledList():ArrayCollection {
			var disabledList:ArrayCollection = new ArrayCollection;
			var i:int = 0;
			var date:Date;
			// １か月目
			for ( i = 0; i < _calendar.disabledList.length; i++ ) {
				date = _calendar.disabledList.getItemAt(i) as Date;
				disabledList.addItem(date);
			}
			return disabledList;
		}

		/**
		 * 平日リスト設定
		 * (配列の中身は Date 型を設定する)
		 *
		 * @param weekdayList ArrayCollection
		 */
		public function set weekdayList(weekdayList:ArrayCollection):void {
			if ( weekdayList != null )
				_weekdayList = weekdayList;
		}
		/**
		 * 休日リスト設定
		 * (配列の中身は Date 型を設定する)
		 *
		 * @param holidayList ArrayCollection
		 */
		public function set holidayList(holidayList:ArrayCollection):void {
			if ( holidayList != null )
				_holidayList = holidayList;
		}
		/**
		 * 土曜日休み設定
		 *
		 * @param saturdayIsHoliday true:休みである, false:休みでない
		 */
		public function set saturdayIsHoliday(saturdayIsHoliday:Boolean):void {
			_saturdayIsHoliday = saturdayIsHoliday;
		}
		/**
		 * 日曜日休み設定
		 *
		 * @param sundayIsHoliday true:休みである, false:休みでない
		 */
		public function set sundayIsHoliday(sundayIsHoliday:Boolean):void {
			_sundayIsHoliday = sundayIsHoliday;
		}
	}
}