/**
* The MIT License
*
* Copyright (c) 2013 saka-en, http://www.saka-en.com/
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/
package datepicker
{
	import mx.controls.DateField;

	public class Util
	{
		/**
		 * 指定された Date 型より、その月の最終日を返します。
		 *
		 * @param date Date 型
		 * @return Date 型
		 */
		public static function lastDayOfThisMonth(date:Date):Date {
			const milliSecondsPerDay:int = 1000 * 60 * 60 * 24;
			var year:Number = date.fullYear;
			var month:Number = date.month + 1;
			var addDate:Date = new Date(year, month, 1, 00, 00);
			var dueDate:Date = new Date(addDate.getTime() + (-1 * milliSecondsPerDay));
			return dueDate;
		}

		/**
		 * 指定されたパターン文字列の日付文字列を Date オブジェクトにして返します。
		 * Date オブジェクトとして有効でない場合は null を返します。
		 *
		 * @param value 日付を表す文字列
		 * @param format 日付を表す文字列のパターン書式
		 * @return 日付を表す文字列の Date オブジェクト
		 */
		public static function toDate(value:String, format:String = "YYYY/MM/DD"):Date {
			return DateField.stringToDate(value, format);
		}

		/**
		 * 指定された Date型 より 曜日を返します。
		 *
		 * @param date 日付型
		 * @param 言語: "ja_JP": 日本語, "en_US": 英語
		 * @return 日付文字列( 2013/04/08 など )
		 */
		public static function getDateOfTheWeek(date:Date, lang:String="ja_JP"):String {
			if ( date == null ) return "";
			var dateOfTheWeek:Array;
			if ( lang == "ja_JP" )
				dateOfTheWeek = ["日","月","火","水","木","金","土"];
			else if ( lang == "en_US" )
				dateOfTheWeek = ["S","M","T","W","T","F","S"];
			else
				dateOfTheWeek = ["日","月","火","水","木","金","土"];
			return dateOfTheWeek[date.day];
		}

		/**
		 * 指定された Date 型 に 追加月を加算した Date 型を返します。
		 *
		 * @param date 対象日付
		 * @param addM 追加月( 前月:-1, 翌月:1 など )
		 * @return Date 型
		 */
		public static function addMonth(date:Date, addM:Number=1):Date {
			if ( date == null ) return null;
			var dueDate:Date = new Date(date.fullYear, date.month + addM, date.date);
			return dueDate;
		}

		/**
		 * 引数に指定された 2 つのオブジェクトを比較します。
		 * 2 つの引数が null でなく、同じ文字シーケンスを表す String オブジェクトである場合にだけ、結果は true になります。
		 *
		 * @param value 比較するオブジェクト
		 * @param anotherValue 比較される、もうひとつオブジェクト
		 * @return String が等しい場合は true 、そうでない場合は false
		 */
		public static function equals(value:String, anotherValue:String):Boolean {
			if ( value == null || anotherValue == null ) return false;
			if ( value == anotherValue )
				return true;
			else
				return false;
		}
	}
}